plugin simpleMeshMod Vu_Parse_Quads
	name:"Vu_Parse_Quads"
	classID:#(0x3D3BEC97, 0xD238D3)
	category:"Max Creation Graph"
(
	local pluginDefinitionFile -- the plugin definition file
	local pluginInstance -- the plugin instance created from the plugin definition
	local _meshWrapper -- the dotnet wrapper around the stack mesh pointer
	local argsType = dotNetClass "System.Object[]" -- used when creating args array
	local dotnet_ValueToDotNetObject = dotnet.ValueToDotNetObject
	local tmHandler

	-- function wrappers
	local _meshWrapper_SetValue
	local pluginInstance_UpdateMesh

	-- value wrapper local and function wrapper declarations
	local PickUnwrapperWrapper, PickUnwrapperWrapper_SetValue
	local meshWrapper, meshWrapper_SetValue
	local Pick_CloneWrapper, Pick_CloneWrapper_SetValue


	parameters main rollout:params
	(
		Show_Emitter ui:Show_Emitter default:True type:#boolean animatable:true
		Populate_Clone ui:Populate_Clone default:False type:#boolean animatable:true
		Reverse ui:Reverse default:False type:#boolean animatable:true
		Grow ui:Grow default:True type:#boolean animatable:true
		UseUnwrap ui:UseUnwrap default:False type:#boolean animatable:true
		PickUnwrapper ui:PickUnwrapper type:#node useNodeOsValidity:true useNodeTmValidity:true
		Use_Emitter_Step ui:Use_Emitter_Step default:False type:#boolean animatable:true
		OffsetFirstTri ui:OffsetFirstTri default:False type:#boolean animatable:true
		Face_ ui:Face_ default:0 type:#integer animatable:true
		Emitter_Step_ ui:Emitter_Step_ default:1 type:#integer animatable:true
		CloneStep_ ui:CloneStep_ default:1 type:#integer animatable:true
		Pick_Clone ui:Pick_Clone type:#node useNodeOsValidity:true useNodeTmValidity:true
		Use_WTF_ ui:Use_WTF_ default:False type:#boolean animatable:true
		Flip ui:Flip default:False type:#boolean animatable:true
		Up_Axis ui:Up_Axis default:2 type:#integer animatable:true
		Rot_Y ui:Rot_Y default:0 type:#float animatable:true
		Rot_X ui:Rot_X default:0 type:#float animatable:true
		Rot_Z ui:Rot_Z default:0 type:#float animatable:true
		WTFY ui:WTFY default:0 type:#float animatable:true
		WTFX ui:WTFX default:0 type:#float animatable:true
		WTFZ ui:WTFZ default:0 type:#float animatable:true
		Surface_Offset ui:Surface_Offset default:0 type:#float animatable:true
		_dummy type:#boolean -- this exists to enable easy invalidation of the object        
		pluginGraph type:#filename assettype:#MaxCreationGraph readOnly:true enumAsAsset:true 
        pluginGraphDependencies type:#filenametab assettype:#MaxCreationGraph readOnly:true enumAsAsset:true tabSize:0 tabSizeVariable:true
    ) 

rollout params "Parameters"
	(
		checkbox Grow "Grow" checked:Grow.state
		spinner Face_ "Face:" range:[0, 9999999, 0] type:#integer

		checkbox OffsetFirstTri "OffsetFirstTri" checked:OffsetFirstTri.state
		checkbox Reverse "Reverse" checked:Reverse.state
		checkbox Use_Emitter_Step "Use Emitter Step" checked:Use_Emitter_Step.state
		spinner Emitter_Step_ "Emitter Step:" range:[1, 9999, 1] type:#integer

		label lab14 "Note: Emitter/Clone Step are" align:#left
		label lab15 "independent, ie. if you use" align:#left
		label lab16 "Emitter Step, it does NOT" align:#left
		label lab17 "affect where you can or can't" align:#left
		label lab18 " make a clone (Face No. does)" align:#left

	group "Clone on Quad"
	(
		pickbutton Pick_Clone "Pick Clone" autodisplay:true type:#node
		slider Up_Axis "Up Axis" range:[0, 2, 2] type:#integer
		label lab13 "X                  Y                 Z" align:#left
		checkbox Flip "Flip" checked:Flip.state
		spinner Surface_Offset "Surface Offset" range:[-99999, 99999, 0] type:#float

		spinner Rot_X "Rot X" range:[-99999, 99999, 0] type:#float
		spinner Rot_Y "Rot Y" range:[-99999, 99999, 0] type:#float
		spinner Rot_Z "Rot Z" range:[-99999, 99999, 0] type:#float

		checkbox Use_WTF_ "Use WTF ?" checked:Use_WTF_.state
		spinner WTFX "WTFX" range:[-99999, 99999, 0] type:#float
		spinner WTFY "WTFY" range:[-99999, 99999, 0] type:#float
		spinner WTFZ "WTFZ" range:[-99999, 99999, 0] type:#float

		checkbox Populate_Clone "Populate Clone" checked:Populate_Clone.state
		spinner CloneStep_ "Clone Step:" range:[1, 9999, 1] type:#integer

		checkbox Show_Emitter "Show Emitter" checked:Show_Emitter.state

	)

	group "Experimental (slow!!!)" 
	(
		label lab21 "Use a plane as  Unwrapper" align:#left
		label lab22 "then tweak the res and" align:#left
		label lab23 "dimensions of Unwrapper." align:#left
		label lab24 "Then add Unwrap UVW mod" align:#left
		label lab25 "to see results..." align:#left
		checkbox UseUnwrap "Use Unwrapper" checked:False
		pickbutton PickUnwrapper "PickUnwrapper" autodisplay:true type:#node
	)
	)

rollout inforoll "About"
	(
		Group "Vu ParseQuads Info"
		(
			label lab91 "Author: Vu Nguyen" align:#left
			label lab92 "(c) Vu Nguyen 2015" align:#left
			label lab94 "Version: 2.05.00" align:#left
			label lab95 "Email: vusta@hotmail.com" align:#left
		)

	)

	fn initialize = 
	(
		pluginDefinitionFile = @"C:/Users/Honey/Autodesk/3ds Max 2016/Max Creation Graph/Tools/Vu_Parse_Quads.maxtool"
		local c = dotNetClass "Viper3dsMaxBridge.ModifierPluginInstance"
		local cid = this.classId
		local refMakerWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.ReferenceMaker"
		refMakerWrapper = refMakerWrapperClass._CreateWrapper this
		pluginInstance = c.CreateInstance cid[1] cid[2] pluginDefinitionFile refMakerWrapper
		pluginInstance_UpdateMesh = pluginInstance.UpdateMesh
		local meshWrapperClass = dotnetclass "Autodesk.Max.MaxPlus.Mesh"
		_meshWrapper = meshWrapperClass._CreateWrapper undefined
		_meshWrapper_SetValue = _meshWrapper._SetValue
		pluginGraph = pluginDefinitionFile
		pluginGraphDependencies = #(@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\DivideByTwo.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\Lesser.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\MultiplyByTwo.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\PlusOne.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\MinusOne.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\Array2.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\SkipOne.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\ArrayFromToStep.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\SelectByIndex.maxcompound",@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\ChooseAxis.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\Array3.maxcompound",@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\VuQuadNormal.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\AverageVector.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\Sum.maxcompound",@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Tools\Downloads\MultiSlice\Compounds\VectorValueAtIndex.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\VectorAsArray.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\Array3.maxcompound",@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\AndAlso3.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\OffsetXY.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\Vector3FromXY.maxcompound",@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\RotationBetweenUnitVectors.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\RotateMesh.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\DeformMeshPoints.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\Origin.maxcompound",@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Compounds\VuQuadMidpoint.maxcompound",@"C:\Users\Honey\Autodesk\3ds Max 2016\Max Creation Graph\Tools\Downloads\MultiSlice\Compounds\VectorValueAtIndex.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\VectorAsArray.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\OffsetVector.maxcompound",@"C:\Program Files\Autodesk\3ds Max 2016\MaxCreationGraph\Compounds\TransformMesh.maxcompound")

		-- value wrapper local initializations
		local PickUnwrapperWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.INode"
		PickUnwrapperWrapper = PickUnwrapperWrapperClass._CreateWrapper (undefined)
		PickUnwrapperWrapper_SetValue = PickUnwrapperWrapper._SetValue
		local meshWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.Mesh"
		meshWrapper = meshWrapperClass._CreateWrapper (undefined)
		meshWrapper_SetValue = meshWrapper._SetValue
		local Pick_CloneWrapperClass = dotNetClass "Autodesk.Max.MaxPlus.INode"
		Pick_CloneWrapper = Pick_CloneWrapperClass._CreateWrapper (undefined)
		Pick_CloneWrapper_SetValue = Pick_CloneWrapper._SetValue

	)

	fn updateMesh = 
	(

		if (pluginInstance == undefined) then ( return undefined )
		local args = dotnet_ValueToDotNetObject #(Show_Emitter, Populate_Clone, Reverse, Grow, UseUnwrap, PickUnwrapperWrapper, meshWrapper, Use_Emitter_Step, OffsetFirstTri, Face_, Emitter_Step_, CloneStep_, Pick_CloneWrapper, Use_WTF_, Flip, Up_Axis, Rot_Y, Rot_X, Rot_Z, WTFY, WTFX, WTFZ, Surface_Offset) argsType
		pluginInstance_UpdateMesh (currentTime as integer) _meshWrapper args
	)
	
	on modifyMesh do
	(
		try 
		(
			_meshWrapper_SetValue mesh
			PickUnwrapperWrapper_SetValue PickUnwrapper
			Pick_CloneWrapper_SetValue Pick_Clone
			meshWrapper_SetValue mesh

			updateMesh()
			_meshWrapper_SetValue undefined

		)
		catch()
		ok
	)

	on create do 
	(
		initialize()
	)

	on clone fromObj do 
	(
		initialize()
	)

	on load do 
	(
		initialize()
	)

	on update do 
	(
		initialize()
	)
)
